import java.util.*;
/**
* This class manages one or more symbol tables for a compiler.
* Tables are indexed from 0.
*/
public class SymbolTable {
	/**
	* Construct a new SymbolTable manager with one table.
	*/
	public SymbolTable() {
		this(1);
	}
	/**
	* Construct a new SymbolTable manager with n tables.
	* @param n number of distinct tables to manage.
	*/
	public SymbolTable(int n) {
		tables = new HashMap[n];
		for (int i=0; i<n; i++) {
			tables[i] = new HashMap();
		}
	}
	/**
	* Add a symbol to table[0].
	* @param key the key (the symbol label) to use for this Symbol
	* @param s the Symbol to add.
	* @return a reference to the Symbol that was already in the table
	* under this key, or null if this is a new entry.
	*/
	// public Symbol putSymbol(String key, Symbol s) {
		// return this.putSymbol(0,key,s);
	// }
	/**
	* Add a symbol to table[k].
	* @param k the table number to use.  Must be 0 to n-1, where n is
	* the number of defined tables.
	* @param key the key (the symbol label) to use for this Symbol
	* @param s the Symbol to add.
	* @return a reference to the Symbol that was already in the table
	* under this key, or null if this is a new entry.
	*/
	public Symbol putSymbol(int k,String key, Symbol s) {
		return (Symbol)tables[k].put(key,s);
	}
	/**
	* Get a symbol from table[0].
	* @param key the key (the symbol label) used for this Symbol
	* @return a reference to the Symbol that is in the table under
	* this key, or null if the table contains no mapping for this key.
	*/
	// public Symbol getSymbol(String key) {
		// return this.getSymbol(0,key);
	// }
	/**
	* Get a symbol from table[k].
	* @param k the table number to use.  Must be 0 to n-1, where n is
	* the number of defined tables. 
	* @param key the key (the symbol label) used for this Symbol
	* @return a reference to the Symbol that is in the table under
	* this key, or null if the table contains no mapping for this key.
	*/
	public Symbol getSymbol(int k,String key) {
		return (Symbol)tables[k].get(key);
	}
	/**
	* Check to see if a symbol is in table[k].
	* @param k the table number to use.  Must be 0 to n-1, where n is
	* the number of defined tables. 
	* @param key the key (the symbol label) used for this Symbol
	* @return true or false depending on whether or not the symbol has
	* already been declared.
	*/
	public boolean isDeclared(int k,String key) {
		return tables[k].containsKey(key);
	}
	/**
	* Check to see if a symbol is in any of the defined tables.
	* @param key the key (the symbol label) used for this Symbol
	* @return true or false depending on whether or not the symbol has
	* already been declared.
	*/
	public boolean isDeclared(String key) {
		for (int i=0; i<tables.length; i++) {
			if (this.isDeclared(i,key)) return true;
		}
		return false;
	}
	
	//-----------------------------------------------------------------------
	
	private HashMap[] tables;
	
}
